@@ -1,7 +1,7 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2014 by Cenon GmbH
+ * Copyright (C) 1995-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
@@ -75,7 +75,7 @@
 @implementation App
 
 /*
- * modified: 2006-12-11
+ * modified: 2015-09-02 ("importMoveToOrigin" = YES added)
  * Initializes the defaults.
  */
 + (void)initialize
@@ -132,8 +132,9 @@
     [registrationDict setObject:@"25.4"      forKey:@"dxfRes"];
     [registrationDict setObject:@"NO"        forKey:@"psFlattenText"];
     [registrationDict setObject:@"NO"        forKey:@"psPreferArcs"];
-    [registrationDict setObject:@"NO"        forKey:@"colorToLayer"];
-    [registrationDict setObject:@"NO"        forKey:@"fillObjects"];
+    [registrationDict setObject:@"NO"        forKey:@"colorToLayer"];       // import
+    [registrationDict setObject:@"NO"        forKey:@"fillObjects"];        // import
+    [registrationDict setObject:@"YES"       forKey:@"importMoveToOrigin"]; // import
     //[registrationDict setObject:@"NO"        forKey:@"icutFillClosedPaths"];
     //[registrationDict setObject:@"NO"        forKey:@"icutOriginUL"];
 
@@ -182,7 +183,7 @@
 
     if (cdir && [cdir length])
         path = cdir;
-    else
+    else    // FIXME: Yosemite: directory is not shared any more between OpenPanels
         path = (haveOpenedDocument ? [((NSOpenPanel*)[NSOpenPanel openPanel]) directory]
                                    : NSHomeDirectory());
     if (!path)
@@ -362,10 +363,11 @@
             if ( NSRunAlertPanel(@"", NSLocalizedString(@"Cenon User-Library will be moved from old location\n \"%@\"\nto new location\n \"%@\".", NULL),
                                  OK_STRING, CANCEL_STRING, nil, oldPath, path) == NSAlertDefaultReturn )
             {
-                if ( [fileManager respondsToSelector:@selector(moveItemAtPath:toPath:error:)] ) // >= 10.5
-                    [fileManager moveItemAtPath:oldPath toPath:path error:NULL];    // mv oldPath -> path
-                else                                                                // <  10.5
-                    [fileManager movePath:oldPath toPath:path handler:nil];
+                [fileManager movePath:oldPath toPath:path error:NULL];
+                //if ( [fileManager respondsToSelector:@selector(moveItemAtPath:toPath:error:)] ) // >= 10.5
+                //    [fileManager moveItemAtPath:oldPath toPath:path error:NULL];    // mv oldPath -> path
+                //else                                                                // <  10.5
+                //    [fileManager movePath:oldPath toPath:path handler:nil];
             }
         }
         else if ( [fileManager fileExistsAtPath:oldPath] )  // it's in both locations
@@ -394,11 +396,11 @@
             /* copy Cenon/Projects/.dir.tiff */
             from = vhfPathWithPathComponents(localLibrary(), @"Projects", @".dir.tiff", nil);
             to   = vhfPathWithPathComponents(path,           @"Projects", @".dir.tiff", nil);
-            [fileManager copyPath:from toPath:to handler:nil];
+            [fileManager copyPath:from toPath:to error:NULL];
             /* copy Cenon/dir.tiff */
             from = vhfPathWithPathComponents(localLibrary(), @".dir.tiff", nil);
             to   = vhfPathWithPathComponents(path,           @".dir.tiff", nil);
-            [fileManager copyPath:from toPath:to handler:nil];
+            [fileManager copyPath:from toPath:to error:NULL];
         }
     }
     /* create symbolic link from FOME/Documents/Cenon -> HOME/Library/../Cenon
@@ -460,7 +462,8 @@
     document = [Document new];
 }
 
-/* modified: 2014-07-23 (cenon format added)
+/* modified: 2015-09-02 (Gerber: -setOrigin: added, PDF: -moveToOrigin: added)
+ *           2014-07-23 (cenon format added)
  *           2011-09-16 (icut format added)
  *           2010-09-10 (defaultManager renamed to fileManager)
  *           2009-02-25 (fileNameLC)
@@ -470,8 +473,10 @@
     NSString        *path, *name;
     NSFileManager   *fileManager = [NSFileManager defaultManager];
     NSString        *fileNameLC = [fileName lowercaseString];
+    //NSString        *ext = [[fileName pathExtension] lowercaseString];
+    DocView         *view = [[self currentDocument] documentView];
 
-    if ( [fileNameLC hasSuffix:@".cenon"] || [fileNameLC hasSuffix:@".cen"] )
+    if ( [fileNameLC hasSuffix:@".cenon"] || [fileNameLC hasSuffix:@".cen"] )   // [ext isCaseInsensitiveLike:DOCUMENT_EXT]
     {   NSString        *fileDirectory = fileName;
         NSDictionary	*plist = nil;
 
@@ -488,7 +493,7 @@
     //if ( [fileName rangeOfString:@".dxf" options:NSCaseInsensitiveSearch].length )
     //if ( [fileName compare:@".dxf" options:NSAnchoredSearch|NSBackwardsSearch|NSCaseInsensitiveSearch] )
     else if ( [fileNameLC hasSuffix:@".dxf"] )
-    {	id	dxfImport = [[DXFImportSub allocWithZone:[self zone]] init];
+    {	id      dxfImport = [[DXFImportSub allocWithZone:[self zone]] init];
         NSData	*data = [NSData dataWithContentsOfFile:fileName];	// get data object
 
         [dxfImport setRes:Prefs_DXFRes];
@@ -588,10 +593,12 @@
                 }
             }
         }
+        [gerberImport setOrigin:((view) ? [[view origin] pointWithNum:0]
+                                        : NSMakePoint(MMToInternal(10.0), MMToInternal(10.0)))];    // Prefs_ImportMoveToOrigin
         list = [[gerberImport importGerber:data] retain];	// get list of graphic-objects from import
         [gerberImport release];
     }
-    else if ( [fileNameLC hasSuffix:@".drl"] || [fileNameLC hasSuffix:@".din"] )
+    else if ( [fileNameLC hasSuffix:@".drl"] || [fileNameLC hasSuffix:@".din"] || [fileNameLC hasSuffix:@".nc"] )
     {   id          dinImport;
         NSData      *data;
         NSString    *devFileName;
@@ -665,6 +672,7 @@
                 NSRunAlertPanel(@"", PSIMPORT_INSTALLGS_STRING, OK_STRING, nil, nil, nil);
         }
 #endif
+        [psImport moveToOrigin:Prefs_ImportMoveToOrigin];
         [psImport preferArcs:Prefs_PSPreferArcs];
         [psImport flattenText:Prefs_PSFlattenText];
         list = [[psImport importPDFFromFile:fileName] retain];
@@ -691,14 +699,14 @@
     NSFileManager	*fileManager = [NSFileManager defaultManager];
 
     /* Adobe Illustrator */
-    if ( [fileName hasSuffix:@".ai"] )
+    if ( [fileName hasSuffix:@".ai"] || [fileName hasSuffix:@".AI"] )
     {   NSString	*string, *header;
         NSRange		range;
+        NSError     *error = nil;
 
-        // FIXME: this is actually covered by (VHFStringAdditions +stringWithContentsOfFile:) ???
-        string = [NSString stringWithContentsOfFile:fileName];   // get file
+        string = [NSString stringWithContentsOfFile:fileName error:&error];   // get file
         range = [string rangeOfString:@"%%BeginResource"];
-        if ( !range.length )	// no ps-header
+        if ( ! range.length )	// no ps-header
         {
             path = [[[NSBundle bundleForClass:[PSImport class]] resourcePath]
                     stringByAppendingPathComponent:AI_HEADER];
@@ -706,7 +714,7 @@
             {   NSRunAlertPanel(@"", CANTLOADFILE_STRING, OK_STRING, nil, nil, path);
                 return nil;
             }
-            header = [NSString stringWithContentsOfFile:path];	// get file
+            header = [NSString stringWithContentsOfFile:path error:NULL];   // get file
 
             string = [header stringByAppendingString:string];
         }
@@ -717,7 +725,7 @@
 
     /* import file */
     psImport = [[[PSImportSub allocWithZone:[self zone]] init] autorelease];
-    //[psImport moveToOrigin:NO];
+    [psImport moveToOrigin:Prefs_ImportMoveToOrigin];
     [psImport preferArcs:Prefs_PSPreferArcs];
     [psImport flattenText:Prefs_PSFlattenText];
 #ifdef __APPLE__    // check, if gs is installed (Linux has it installed anyway, OpenStep uses DPS)
@@ -732,6 +740,7 @@
 - (BOOL)openFile:(NSString *)fileName
 {   id              list = nil;
     NSString        *path;
+    NSString        *ext = [fileName pathExtension];
     NSFileManager   *fileManager = [NSFileManager defaultManager];
 
     if (!appIsRunning)	// give application a chance to load modules first
@@ -743,7 +752,7 @@
     document = nil;
     if ( ![fileManager fileExistsAtPath:fileName] )
         return NO;
-    if ( [fileName hasSuffix:DOCUMENT_EXT] )
+    if ( [ext isCaseInsensitiveLike:DOCUMENT_EXT] || [ext isCaseInsensitiveLike:@"cen"] )
     {	int	i;
 
         for (i=[[self windows] count]-1; i>=0; i--)
@@ -764,7 +773,7 @@
     else if ( (list = [self listFromFile:fileName]) )
     {
         /* Fonts: special treatment for fonts */
-        if ( [fileName hasSuffix:@".font"] || [fileName hasSuffix:@".pfa"] || [fileName hasSuffix:@".pfb"] )
+        if ( [ext isCaseInsensitiveLike:@".font"] || [ext isCaseInsensitiveLike:@".pfa"] || [ext isCaseInsensitiveLike:@".pfb"] )
         {
             document = [Document new];
             [self setCurrentDocument:nil];
@@ -776,6 +785,8 @@
         else
         {
             document = [Document newFromList:list];
+            [document setName:[[fileName lastPathComponent] stringByDeletingPathExtension]  // set name of imported file
+                 andDirectory:[fileName stringByDeletingLastPathComponent]];                // preserve open directory
             [self setCurrentDocument:nil];
         }
         return YES;
@@ -793,10 +804,16 @@ typedef enum
     IMPORTTO_EXISTINGLAYERS = 2
 } ImportToLayerSelection;
 - (void)import:sender
-{   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
-                                              @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
-                                              @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                              @"font", @"pfa", @"din", @"drl", @"svg", @"cut", @"icut", nil];
+{   NSArray     *fileTypes = [NSArray arrayWithObjects: @"cenon", @"cen",
+                                                        @"hpgl", @"hgl", @"plt",
+                                                        @"ai", @"eps", @"ps", @"pdf",
+                                                        @"dxf",
+                                                        @"gerber", @"ger",
+                                                        @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
+                                                        @"font", @"pfa",
+                                                        @"din", @"drl", @"nc",
+                                                        @"svg",
+                                                        @"cut", @"icut", nil];
     NSString    *fileName;
     id          openPanel = [NSOpenPanel openPanel];
     static      NSString	*openDir = @"";
@@ -884,19 +901,26 @@ typedef enum
  *           2011-12-03 ("cut", "icut" added)
  */
 - (void)openDocument:sender
-{   NSArray     *fileTypes = [NSArray arrayWithObjects:@"hpgl", @"hgl", @"plt", @"ai",
-                                  @"dxf", @"eps", @"ps", @"pdf", @"gerber", @"ger",
-                                  @"cenon", @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
-                                  @"font", @"pfa", @"din", @"drl", @"svg", @"cut", @"icut", nil];
+{   NSArray     *fileTypes = [NSArray arrayWithObjects: @"cenon", @"cen",
+                              @"hpgl", @"hgl", @"plt",
+                              @"ai", @"eps", @"ps", @"pdf",
+                              @"dxf",
+                              @"gerber", @"ger",
+                              @"tiff", @"tif", @"jpg", @"jpeg", @"gif", @"png",
+                              @"font", @"pfa",
+                              @"din", @"drl", @"nc",
+                              @"svg",
+                              @"cut", @"icut", nil];
     NSArray         *fileNames;
     id              openPanel = [NSOpenPanel openPanel];
     int             i, cnt;
-    static NSString *openDir = @"";
+    static NSString *openDir = nil;
 
     if (openPanelAccessory)
         [openPanel setAccessoryView:[openPanelAccessory retain]];
     [openPanel setAllowsMultipleSelection:YES];
-    [openPanel setDirectory:openDir];
+    if ( [openDir length] )
+        [openPanel setDirectory:openDir];
     [openPanel setAllowedFileTypes:fileTypes];  // FIXME, Apple: this method doesn't work, we stay with the deprecated modal method (2012-03-13)
 
     if ( [openPanel runModalForDirectory:openDir file:@"" types:fileTypes] )
@@ -952,20 +976,26 @@ typedef enum
 }
 
 - (void)changeSaveType:sender
-{   id	savePanel = [sender window];
+{   NSSavePanel *savePanel = (NSSavePanel*)[sender window];
+    NSArray     *array = nil;
 
     switch ([sender indexOfSelectedItem])
     {
-        case 0: [savePanel setRequiredFileType:DOCUMENT_EXT]; break;
-        case 1: [savePanel setRequiredFileType:EPS_EXT];      break;
-        case 2: [savePanel setRequiredFileType:GERBER_EXT];   break;
-        case 3: [savePanel setRequiredFileType:DXF_EXT];      break;
-        case 4: [savePanel setRequiredFileType:HPGL_EXT];     break;
-        case 5: [savePanel setRequiredFileType:TIFF_EXT];     break;
-        case 6: [savePanel setRequiredFileType:FONT_EXT];     break;
-        case 7: [savePanel setRequiredFileType:DIN_EXT];      break;
-        default: NSLog(@"App, changeFileType: Unknown file type");
+        // TODO: get document types from info property list, bundle resources, or whereever they are hiding
+        case 0: array = [NSArray arrayWithObjects:DOCUMENT_EXT, @"cen", nil];           break;
+        case 1: array = [NSArray arrayWithObjects:EPS_EXT,      @"ps",  nil];           break;
+        case 2: array = [NSArray arrayWithObjects:GERBER_EXT,   @"ger", nil];           break;
+        case 3: array = [NSArray arrayWithObjects:DXF_EXT, nil];                        break;
+        case 4: array = [NSArray arrayWithObjects:HPGL_EXT,     @"hgl", @"plt", nil];   break;
+        case 5: array = [NSArray arrayWithObjects:TIFF_EXT,     @"tif", nil];           break;
+        case 6: array = [NSArray arrayWithObjects:FONT_EXT, nil];                       break;
+        case 7: array = [NSArray arrayWithObjects:DIN_EXT,      @"nc", @"drl", nil];    break;
+        default: NSLog(@"App, changeFileType: Unknown file type");                      return;
     }
+    if ( [savePanel respondsToSelector:@selector(setAllowedFileTypes:)] )
+        [savePanel setAllowedFileTypes:array];
+    else
+        [savePanel setRequiredFileType:[array objectAtIndex:0]];
 }
 
 - (void)revertToSaved:sender
@@ -1018,7 +1048,10 @@ typedef enum
      */
     [savePanel setAccessoryView:[savePanelAccessory retain]];
     //[self changeSaveType:spaFormatPopUp];
-    [savePanel setRequiredFileType:DOCUMENT_EXT];
+    if ( [savePanel respondsToSelector:@selector(setAllowedFileTypes:)] )
+        [savePanel setAllowedFileTypes:[NSArray arrayWithObjects:DOCUMENT_EXT, @"cen", nil]];
+    else
+        [savePanel setRequiredFileType:DOCUMENT_EXT];
     [spaFormatPopUp selectItemAtIndex:0];
 
     //[savepanel setTitle:@"Save As"];
