@@ -1,13 +1,15 @@
 /* DXFExportSub.m
  * subclass of DXFExport
  *
- * Copyright (C) 2002-2014 by Cenon GmbH
+ * Copyright (C) 2002-2015 by Cenon GmbH
  * Author:   Ilonka Fleischmann
  *
  * created:  2002-04-25
- * modified: 2010-04-03 (-exportArc: export -360 angle correctly (not 0))
+ * modified: 2015-06-09 (-exportPath: don't use [VPath -getLastObjectOfSubPath:] method for open paths)
+ *                      replace deprecated NSFileManager methods
+ *           2010-04-03 (-exportArc: export -360 angle correctly (not 0))
  *
- * This file is part of the vhf Export Library.
+ * This file is part of the Cenon Export Library.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the Cenon Public License as
@@ -35,7 +37,8 @@
 
 #include <AppKit/AppKit.h>
 #include <VHFShared/vhfCommonFunctions.h>
-#include <VHFShared/VHFStringAdditions.h> // stringByReplacing -> eps export
+#include <VHFShared/VHFStringAdditions.h>   // stringByReplacing -> eps export
+#include <VHFShared/vhfSystemAdditions.h>   // [NSFileManager methods]
 #include "App.h"
 #include "DXFExportSub.h"
 #include "Graphics.h"
@@ -109,7 +112,7 @@
 
 - (void)exportCurve:(VCurve*)g
 {   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-    VPath	*path;
+    VPath               *path;
 
     if ( (path = [g flattenedObjectWithFlatness:0.1]) )
     {   //int	i, cnt = [[path list] count];
@@ -129,23 +132,24 @@
 
 - (void)exportText:(VText*)g
 {   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-    id			text = [g pathRepresentation];
+    id                  text = [g pathRepresentation];
 
     if (Prefs_ExportFlattenText)
-    {   if (text && [text isKindOfClass:[VPath class]])
+    {
+        if (text && [text isKindOfClass:[VPath class]])
            [self exportPath:text];
         else if (text /*&& [text isKindOfClass:[VGroup class]]*/)
             [self exportGroup:text];
     }
     else
     {	//int		mirror = 0; // 0-not 2-x 4-y 6-x&y
-        float		lCnt = 0.0, degree = [g rotAngle], iangle = [[g font] italicAngle]; // cw to x axis
-        float		pS=[[g font] pointSize], desc=[[g font] descender], lH=[g lineHeight];
+        VFloat		lCnt = 0.0, degree = [g rotAngle], iangle = [[g font] italicAngle]; // cw to x axis
+        VFloat		pS = [[g font] pointSize], desc = [[g font] descender], lH = [g lineHeight];
         NSString	*str = [g string];
         NSRect		bounds = [g textBox], fbounds = [[g font] boundingRectForFont]; // bounds
         NSPoint		o = bounds.origin; // groundline left
         NSScanner	*scanner = [NSScanner scannerWithString:[g string]];
-        //float	lS = [g lineSpacing]; // pS=[[g font] pointSize];
+        //VFloat	lS = [g lineSpacing]; // pS=[[g font] pointSize];
 
         // jede zeile einzeln max 256 asci zeichen
         while (![scanner isAtEnd])
@@ -165,24 +169,23 @@
 }
 
 - (void)exportPath:(VPath*)path
-{   int		i, cnt=[[path list] count];
-    int		closed = ([path filled]) ? (1) : (0); // lines and arcsegments
-    int		begIx=0, endIx=0;
+{   int		i, cnt = [[path list] count];
+    BOOL	closed = ([path filled]) ? YES : NO;    // lines and arcsegments
+    int		begIx = 0, endIx = 0;
 
-
-    for (i=0; i<cnt; i++)
+    for (i=0; i < cnt; i++)
     {   id	g = [[path list] objectAtIndex:i];
 
-
-        if ( !i || i>endIx ) // each subpath itself !
+        if ( !i || i > endIx )  // each subpath itself !
         {
             if (i) // close polyline
                 [self writePolyLineMode:NO :[path width] :closed];
             [self writePolyLineMode:YES :[path width] :closed];
 
             begIx = i;
-            //endIx = [self getLastObjectOfSubPath:begIx];
-            endIx = [path getLastObjectOfSubPath:begIx]; //  tolerance:TOLERANCE
+            if ( closed )
+                endIx = [path getLastObjectOfSubPath:begIx]; //  tolerance:TOLERANCE
+            endIx = begIx;
         }
 
         // end point is start of next graphic ! - but open path
@@ -202,7 +205,7 @@
         }
         else if (  [g isKindOfClass:[VRectangle class]] )
         {   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-            VPath	*p;
+            VPath               *p;
 
             if ( (p = [g pathRepresentation]) )
             {   int	j, gCnt = [[p list] count];
@@ -313,7 +316,7 @@
         }
         else if (  [g isKindOfClass:[VCurve class]] )
         {   NSAutoreleasePool	*pool = [NSAutoreleasePool new];
-            VPath		*p;
+            VPath               *p;
 
             if ( (p = [g flattenedObjectWithFlatness:0.1]) )
             {   int	j, gCnt = [[p list] count];
@@ -334,7 +337,7 @@
 }
 
 - (void)exportGroup:(VGroup*)group
-{   int		i, cnt=[[group list] count];
+{   int		i, cnt = [[group list] count];
 
     for (i=0; i<cnt; i++)
     {   VGraphic    *g = [[group list] objectAtIndex:i];
@@ -373,8 +376,10 @@
         return NO;
     }
     /* rename to backup */
-    if (   ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil])
-        || ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if (   ( [fileManager fileExistsAtPath:backupFilename] &&
+            ![fileManager removeFileAtPath:backupFilename error:nil])
+        || ( [fileManager fileExistsAtPath:filename]       &&
+            ![fileManager movePath:filename toPath:backupFilename error:nil]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
@@ -443,15 +448,12 @@
     /* restore backup */
     if (!savedOk)
     {
-        [fileManager removeFileAtPath:filename handler:nil];	// remove what we just started to write
-        [fileManager movePath:backupFilename toPath:filename handler:nil];	// restore backup
+        [fileManager removeFileAtPath:filename error:nil];	// remove what we just started to write
+        [fileManager movePath:backupFilename toPath:filename error:nil];	// restore backup
         NSRunAlertPanel(SAVE_TITLE, CANT_SAVE, nil, nil, nil);
     }
-    else
-    {
-        if (Prefs_RemoveBackups)
-            [fileManager removeFileAtPath:backupFilename handler:nil];
-    }
+    else if (Prefs_RemoveBackups)
+        [fileManager removeFileAtPath:backupFilename error:nil];
     return YES;
 }
 
