@@ -1,11 +1,12 @@
 /* Document.m
  * Cenon document class
  *
- * Copyright (C) 1996-2014 by Cenon GmbH
+ * Copyright (C) 1996-2015 by Cenon GmbH
  * Author: Georg Fleischmann
  *
  * Created:  1996-02-09
- * Modified: 2013-06-29 (+newFromFile: scale to saved scaleFactor)
+ * Modified: 2015-08-25 (-printDocument: 10.8: set paper margins to 0)
+ *           2013-06-29 (+newFromFile: scale to saved scaleFactor)
  *           2013-02-05 (-save  Cenon Version saved with file)
  *           2013-01-02 (-printDocument: remove paper margins from view, which are not ignored any more since OS 10.8)
  *           2011-03-30 (-exportLock, -setExportLock:)
@@ -631,7 +632,7 @@ static NSMutableArray *zoneList = nil;
 }
 #endif
 
-- (void)scale:(float)x :(float)y withCenter:(NSPoint)center
+- (void)scale:(VFloat)x :(VFloat)y withCenter:(NSPoint)center
 {
     NSLog(@"[document scale:x :y ...] is deprecated, use [document scale:NSSize ...");
     [self scale:NSMakeSize(x, y) withCenter:center];
@@ -731,14 +732,12 @@ static NSMutableArray *zoneList = nil;
 
 /* Services menu registrar */
 - (void)registerForServicesMenu
-{
-    static BOOL registered = NO;
-    NSArray *validSendTypes = [[NSArray array] initWithObjects:NSFilenamesPboardType, nil];
+{   static BOOL registered = NO;
+    NSArray     *validSendTypes = [[NSArray array] initWithObjects:NSFilenamesPboardType, nil];
 
     if (!registered)
-    {
-	registered = YES;
-	[NSApp registerServicesMenuSendTypes:validSendTypes returnTypes:nil];
+    {   registered = YES;
+        [NSApp registerServicesMenuSendTypes:validSendTypes returnTypes:nil];
     } 
 }
 - (id)validRequestorForSendType:(NSString *)sendType returnType:(NSString *)returnType
@@ -898,6 +897,8 @@ static NSMutableArray *zoneList = nil;
  * newName or newDirectory can be nil, in which case the name or directory
  * will not be changed (unless one is currently not set, in which case
  * a default name will be used).
+ *  fileName  = "name.ext"
+ *  directory = "/path/"
  */
 - (void)setName:(NSString*)newName andDirectory:(NSString*)newDirectory
 {   static int	untitledCount = 0;
@@ -1085,8 +1086,8 @@ static NSMutableArray *zoneList = nil;
         return NO;
     }
     /* rename to backup */
-    if (    ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil])
-         || ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if (    ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename error:NULL])
+         || ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename error:NULL]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
@@ -1214,14 +1215,14 @@ static NSMutableArray *zoneList = nil;
     /* restore backup */
     if (!savedOk)
     {
-        [fileManager removeFileAtPath:fileDirectory handler:nil];	// remove what we just started to write
-        [fileManager movePath:backupFilename toPath:fileDirectory handler:nil];	// restore backup
+        [fileManager removeFileAtPath:fileDirectory error:NULL];    // remove what we just started to write
+        [fileManager movePath:backupFilename toPath:fileDirectory error:NULL];  // restore backup
         NSRunAlertPanel(SAVE_TITLE, CANT_SAVE, nil, nil, nil);
     }
     else
     {
         if (Prefs_RemoveBackups)
-            [fileManager removeFileAtPath:backupFilename handler:nil];
+            [fileManager removeFileAtPath:backupFilename error:NULL];
         [self setDirty:NO];
     }
     [pool release];
@@ -1254,28 +1255,28 @@ static NSMutableArray *zoneList = nil;
         savepanel = [(App*)NSApp saveAsPanelWithSaveType:FONT_EXT];
 
     if ([savepanel runModalForDirectory:directory file:[name stringByDeletingPathExtension]])
-    {   NSString *path = [savepanel filename];
+    {   NSString    *path = [savepanel filename];
+        NSString    *ext = [path pathExtension];
 
-        if ([[path pathExtension] isEqual:DOCUMENT_EXT])
+        if ( [ext isCaseInsensitiveLike:DOCUMENT_EXT] || [ext isCaseInsensitiveLike:@"cen"] )
         {   [self setName:path];
             return [self save];
         }
         if ( exportLock )   // this file is not supposed to be exported
-        {
-            NSRunAlertPanel(SAVE_TITLE, EXPORTLOCK_STRING, nil, nil, nil);
+        {   NSRunAlertPanel(SAVE_TITLE, EXPORTLOCK_STRING, nil, nil, nil);
             return NO;
         }
-        if ([[path pathExtension] isEqual:EPS_EXT])
+        if (      [ext isCaseInsensitiveLike:EPS_EXT]     || [ext isCaseInsensitiveLike:@"ps"] )
             [self saveEPS:path];
-        else if ([[path pathExtension] isEqual:GERBER_EXT])
+        else if ( [ext isCaseInsensitiveLike:GERBER_EXT]  || [ext isCaseInsensitiveLike:@"ger"] )
             [self saveGerber:path];
-        else if ([[path pathExtension] isEqual:HPGL_EXT])
+        else if ( [ext isCaseInsensitiveLike:HPGL_EXT]    || [ext isCaseInsensitiveLike:@"hgl"] || [ext isCaseInsensitiveLike:@"plt"] )
             [self saveHPGL:path];
-        else if ([[path pathExtension] isEqual:DXF_EXT])
+        else if ( [ext isCaseInsensitiveLike:DXF_EXT])
             [self saveDXF:path];
-        else if ([[path pathExtension] isEqual:FONT_EXT])
+        else if ( [ext isCaseInsensitiveLike:FONT_EXT])
             [self saveFont:path];
-        else if ([[path pathExtension] isEqual:TIFF_EXT])
+        else if ( [ext isCaseInsensitiveLike:TIFF_EXT]    || [ext isCaseInsensitiveLike:@"tif"] )
             [self saveTiff:path];
     }
 
@@ -1879,7 +1880,8 @@ static NSMutableArray *zoneList = nil;
 }
 
 /*
- * modified: 2013-01-02 (10.8: remove paper margins by adjusting print view)
+ * modified: 2015-08-25 (10.8: set paper margins to 0)
+ *           2013-01-02 (10.8: remove paper margins by adjusting print view)
  */
 - (void)printDocument:sender
 {   NSPrintOperation    *op;
@@ -1892,15 +1894,39 @@ static NSMutableArray *zoneList = nil;
     /* set scale to 1.0 */
     scale = [docView scaleFactor];
     visibleRect = [docView visibleRect]; // needed later to calc the center to scroll back
-    oldFrame  = [docView frame];
+    oldFrame    = [docView frame];
     width  = oldFrame.size.width  / scale;
     height = oldFrame.size.height / scale;
 #ifdef __APPLE__
-    /* Apple 10.8: the PrintInfo margins are 0, but there are paper margins PMPaperMargins()
-     * we could create a custom paper with PMPaperCreateCustom() or adjust our print view
+    /* Apple 10.8: the PrintInfo margins are 0, but there are now paper margins PMPaperMargins
+     *             We create a custom paper format and set these margins 0 too.
      */
-    if ( NSAppKitVersionNumber > NSAppKitVersionNumber10_7 )    // Workaround Max OS 10.8: make view smaller by paper margins
-    {   width  -= ([printInfo paperSize].width  - pageBounds.size.width);
+    if ( floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_7 &&    // Workaround Max OS 10.8: remove fucking paper margins
+        [printInfo respondsToSelector:@selector(PMPrintSession)] )
+    {   PMPrintSession  session = [printInfo PMPrintSession];
+        PMPrinter       printer = NULL;
+        PMPageFormat    pageFormat = [printInfo PMPageFormat], newPageFormat = NULL;
+        PMPaper         paper = NULL;
+        PMPaperMargins  margins = {0.0, 0.0, 0.0, 0.0};
+
+        PMSessionGetCurrentPrinter(session, &printer);
+        PMGetPageFormatPaper(pageFormat, &paper);
+        //PMPaperGetMargins(paper, &m);
+        //NSLog(@"paper margins = {%f, %f} - {%f, %f}", m.top, m.left, m.bottom, m.right );
+        PMPaperCreateCustom(printer, CFSTR("Custom paper ID"), CFSTR("Custom paper"),
+                            [printInfo paperSize].width, [printInfo paperSize].height, &margins, &paper);
+        PMCreatePageFormatWithPMPaper(&newPageFormat, paper);
+        PMCopyPageFormat(newPageFormat, pageFormat);
+        [printInfo updateFromPMPageFormat];
+        //PMPaperGetMargins(paper, &m);
+        //NSLog(@"paper margins = {%f, %f} - {%f, %f}", m.top, m.left, m.bottom, m.right );
+        PMRelease(newPageFormat);
+        PMRelease(paper);
+    }
+    /* In case the above fails, we remove the margin from our view */
+    if ( floor(NSAppKitVersionNumber) > NSAppKitVersionNumber10_7 )    // Workaround Max OS 10.8: make view smaller by paper margins
+    {   pageBounds = [printInfo imageablePageBounds];
+        width  -= ([printInfo paperSize].width  - pageBounds.size.width);
         height -= ([printInfo paperSize].height - pageBounds.size.height);
     }
 #endif
@@ -2023,7 +2049,7 @@ static NSMutableArray *zoneList = nil;
             //[self resetScrollers];
             //[view display];
             //[self dirty:self];
-	}
+        }
     } 
 }
 
