@@ -1,7 +1,7 @@
 /* VGraphic.m
  * Graphic object - root class
  *
- * Copyright (C) 1996-2014 by Cenon GmbH
+ * Copyright (C) 1996-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  * WWW:      http://www.cenon.info
  *
@@ -28,8 +28,8 @@
  * with this program; see the file LICENSE. If not, write to Cenon.
  *
  * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
- * eMail: cenon@vhf.de
- * http://www.vhf.biz
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -57,7 +57,7 @@ static int dkgrayRectSize  = 0;
 static int dkgrayRectCount = 0;
 
 
-+ (float)maxKnobSizeWithScale:(float)scaleFactor
++ (VFloat)maxKnobSizeWithScale:(VFloat)scaleFactor
 {
     return (KNOBSIZE+4.0) / scaleFactor;    // 2008-02-25: was 2.0
 }
@@ -688,19 +688,37 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 /* bounds including width and knob size
  * never becomes zero!
  */
-- (NSRect)extendedBoundsWithScale:(float)scaleFactor
+- (NSRect)extendedBoundsWithScale:(VFloat)scaleFactor
 {   NSRect	bRect = [self bounds];
-    float	knobsize = -[VGraphic maxKnobSizeWithScale:scaleFactor]/2.0;
+    VFloat	knobsize = -[VGraphic maxKnobSizeWithScale:scaleFactor]/2.0;
 
     bRect = NSInsetRect(bRect, knobsize, knobsize);
     bRect = NSIntegralRect(bRect);
     return bRect;
 }
 
-/* maximum possible bounds (bounds for all angles of rotation)
+/* maximum possible bounds (bounds for all angles of rotation around center of bounds)
  */
-- (NSRect)maximumBounds
-{   float	dx, dy;
+- (NSRect)maxBoundsWithCenter:(NSPoint)center
+{   VFloat	x, y, w, h, l2;
+    NSRect	rect;
+
+    rect = [self bounds];
+    x = rect.origin.x;
+    y = rect.origin.y;
+    w = rect.size.width;
+    h = rect.size.height;
+    l2 = SqrDistPoints(center, rect.origin);
+    l2 = Max(SqrDistPoints(center, NSMakePoint(x,   y+h)), l2);
+    l2 = Max(SqrDistPoints(center, NSMakePoint(x+w, y)),   l2);
+    l2 = Max(SqrDistPoints(center, NSMakePoint(x+w, y+h)), l2);
+    rect.size.width = rect.size.height = sqrt(l2) * 2.0;
+    rect.origin.x = center.x - rect.size.width  / 2.0;
+    rect.origin.y = center.y - rect.size.height / 2.0;
+    return rect;
+}
+/*- (NSRect)maximumBounds
+{   VFloat	dx, dy;
     NSRect	rect;
 
     rect = [self bounds];
@@ -712,7 +730,7 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
     rect.origin.x -= rect.size.width /2.0;
     rect.origin.y -= rect.size.height/2.0;
     return rect;
-}
+}*/
 
 /* created:		22.10.95
  * modified:	
@@ -911,13 +929,13 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
  */
 - (int)getIntersections:(NSPoint**)ppArray with:g
 {
-    NSLog(@"VGraphic, -getIntersections: not implemented!");
+    NSLog(@"VGraphic, -getIntersections: not implemented for class %@ !", [self className]);
     return 0;
 }
 
 - (void)getPointBeside:(NSPoint*)point :(int)left :(float)dist
 {
-    NSLog(@"VGraphic, -getPointOnSide::: not implemented!");
+    NSLog(@"VGraphic, -getPointOnSide::: not implemented for class %@ !", [self className]);
 }
 
 - uniteWith:(VGraphic*)ug
@@ -932,17 +950,17 @@ static void drawControl( NSPoint origin, BOOL direct, BOOL selectedKnob, float s
 
 - (float)sqrDistanceGraphic:g :(NSPoint*)pg1 :(NSPoint*)pg2
 {
-    NSLog(@"VGraphic, -sqrDistanceGraphic::: not implemented!");
+    NSLog(@"VGraphic, -sqrDistanceGraphic::: not implemented for class %@ !", [self className]);
     return MAXCOORD;
 }
 - (float)sqrDistanceGraphic:g
 {
-    NSLog(@"VGraphic, -sqrDistanceGraphic: not implemented!");
+    NSLog(@"VGraphic, -sqrDistanceGraphic: not implemented for class %@ !", [self className]);
     return MAXCOORD;
 }
 - (float)distanceGraphic:g
 {
-    NSLog(@"VGraphic, -distanceGraphic: not implemented!");
+    NSLog(@"VGraphic, -distanceGraphic: not implemented for class %@ !", [self className]);
     return MAXCOORD;
 }
 
