@@ -1,12 +1,14 @@
 /* VRectangle.m
  * 2-D Rectangle object
  *
- * Copyright (C) 1996-2014 by Cenon GmbH
+ * Copyright (C) 1996-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  * WWW:      http://www.cenon.info
  *
  * created:  1996-09-17
- * modified: 2013-08-13 (-hit:fuzz: rotate p back before let path do the job)
+ * modified: 2015-08-06 (-contourOpenLeftOrRight: new)
+ *           2015-07-30 (-create: add temporary lines to [DocView tmpGraphics], [self setNeedsDisplayInRect:], lockFocus removed)
+ *           2013-08-13 (-hit:fuzz: rotate p back before let path do the job)
  *           2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
  *           2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
  *           2006-02-08
@@ -26,8 +28,8 @@
  * with this program; see the file LICENSE. If not, write to Cenon.
  *
  * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
- * eMail: info@vhf.biz
- * http://www.vhf.biz
+ * eMail: info@Cenon.com
+ * http://www.Cenon.de
  */
 
 #include <AppKit/AppKit.h>
@@ -133,31 +135,35 @@
     return (size.x + size.y) * 2.0;
 }
 
+/* create
+ * modified: 2015-07-30 (add temporary lines to [DocView tmpGraphics], [self setNeedsDisplayInRect:], lockFocus removed)
+ */
 #define CREATEEVENTMASK NSLeftMouseDraggedMask|NSLeftMouseDownMask|NSLeftMouseUpMask|NSPeriodicMask
 - (BOOL)create:(NSEvent *)event in:(DocView*)view
-{   NSRect	viewBounds, gridBounds, drawBounds;
-    NSPoint	start, last, gridPoint, drawPoint, rSize, p, lastPoint = NSZeroPoint, hitPoint;
-    id		window = [view window];
-    VRectangle 	*drawRectangleGraphic;
-    BOOL	ok = YES, dragging = NO, hitEdge = NO, inTimerLoop = NO;
-    VFloat	grid = 1.0 / [view scaleFactor];	// minimum accepted length
-    int		windowNum = [event windowNumber];
-    //	BOOL	alternate = [NXApp alternate];
+{   NSRect          viewBounds, gridBounds, drawBounds;
+    NSPoint         start, last, gridPoint, drawPoint, rSize, p, lastPoint = NSZeroPoint, hitPoint;
+    id              window = [view window];
+    VRectangle      *drawRectangleGraphic;
+    BOOL            ok = YES, dragging = NO, hitEdge = NO, inTimerLoop = NO;
+    VFloat          grid = 1.0 / [view scaleFactor];	// minimum accepted length
+    int             windowNum = [event windowNumber];
+    //BOOL          alternate = [NXApp alternate];
+    NSMutableArray  *tmpGraphics = [[view class] tmpGraphics];
 
-    [[(App*)NSApp inspectorPanel] loadGraphic:self];	/* set the values of the inspector to self */
+    [[(App*)NSApp inspectorPanel] loadGraphic:self];    // set the values of the inspector to self
 
-    start = [view convertPoint:[event locationInWindow] fromView:nil];	/* convert window to view coordinates */
+    start = [view convertPoint:[event locationInWindow] fromView:nil];	// convert window to view coordinates
     hitPoint = start;
-    hitEdge = [view hitEdge:&hitPoint spare:self];	// snap to point
-    gridPoint = [view grid:start];			// set on grid
+    hitEdge = [view hitEdge:&hitPoint spare:self];  // snap to point
+    gridPoint = [view grid:start];                  // set on grid
     if ( hitEdge &&
          ((gridPoint.x == start.x && gridPoint.y == start.y)  ||
           (SqrDistPoints(hitPoint, start) < SqrDistPoints(gridPoint, start))) )
-        start = hitPoint; // we took the nearer one if we got a hitPoint
+        start = hitPoint;   // we took the nearer one if we got a hitPoint
     else
         start = gridPoint;
-    viewBounds = [view visibleRect];			/* get the bounds of the view */
-    [view lockFocus];					/* and lock the focus on view */
+    viewBounds = [view visibleRect];                // get the bounds of the view
+    //[view lockFocus];                             // and lock the focus on view
 
     [self setVertices:start :start];
     drawRectangleGraphic = [[self copy] autorelease];
@@ -171,8 +177,8 @@
     /* now entering the tracking loop
      */
     while ( ((!dragging && [event type] != NSLeftMouseDown) ||
-              (dragging && [event type] != NSLeftMouseUp  )) &&
-            [event type] != NSAppKitDefined && [event type] != NSSystemDefined )
+              (dragging && [event type] != NSLeftMouseUp  ))
+           && [event type] != NSAppKitDefined && [event type] != NSSystemDefined )
     {
         /* Since MouseMoved event is never send we use a periodic event instead */
         if ( [event type] == NSPeriodic )
@@ -186,7 +192,7 @@
             lastPoint = drawPoint;
 
             /* delete line from screen */
-            [view drawRect:gridBounds];
+            [view setNeedsDisplayInRect:gridBounds];    //[view drawRect:gridBounds];
             drawPoint = [view convertPoint:drawPoint fromView:nil];
             if ( ([event type] == NSLeftMouseDragged)&&(!dragging) &&
                  (Diff(origin.x, drawPoint.x) > 3.0*grid || Diff(origin.y, drawPoint.y) > 3.0*grid) )
@@ -196,8 +202,7 @@
                 viewBounds = [view bounds];
             }
             /* fix position to grid */
-            gridPoint = drawPoint;
-            gridPoint = [view grid:gridPoint];
+            gridPoint = [view grid:drawPoint];
             /* snap to point */
             hitPoint = drawPoint;
             hitEdge = [view hitEdge:&hitPoint spare:self];
@@ -213,7 +218,7 @@
             rSize.x = Diff(gridPoint.x, start.x);
             rSize.y = Diff(gridPoint.y, start.y);
             [self setVertices:p :rSize];
-            gridBounds = [self extendedBoundsWithScale:[view scaleFactor]];	// get bounds of the grid line
+            gridBounds = [self extendedBoundsWithScale:[view scaleFactor]]; // get bounds of the grid line
 
             p.x = Min(start.x, drawPoint.x);
             p.y = Min(start.y, drawPoint.y);
@@ -223,9 +228,14 @@
             drawBounds = [drawRectangleGraphic extendedBoundsWithScale:[view scaleFactor]];
             gridBounds  = NSUnionRect(drawBounds, gridBounds);
 
-            if ( NSContainsRect(viewBounds, gridBounds) )		// line inside view ?
-            {   [drawRectangleGraphic drawWithPrincipal:view];
-                [self drawWithPrincipal:view];
+            [tmpGraphics removeAllObjects];
+            if ( NSContainsRect(viewBounds, gridBounds) )   // rectangle inside view ?
+            {
+                if ( drawPoint.x != gridPoint.x || drawPoint.y != gridPoint.y )
+                    [tmpGraphics addObject:drawRectangleGraphic];   // shadow rectangle
+                [tmpGraphics addObject:self];                       // on the grid
+                //[drawRectangleGraphic drawWithPrincipal:view];
+                //[self drawWithPrincipal:view];
             }
             else
                 drawPoint = gridPoint = start;				// else set line invalid
@@ -235,6 +245,7 @@
         event = [NSApp nextEventMatchingMask:CREATEEVENTMASK untilDate:[NSDate distantFuture] inMode:NSEventTrackingRunLoopMode dequeue:YES];
     }
     StopTimer(inTimerLoop);
+    [tmpGraphics removeAllObjects];
 
     last = gridPoint;
 
@@ -246,13 +257,13 @@
         if ([event clickCount] > 1 || [event windowNumber] != windowNum)
             ok = NO;
         else
-            ok = NSMouseInRect(gridPoint , viewBounds , NO);
+            ok = NSMouseInRect(gridPoint, viewBounds , NO);
     }
 
     if ([event type] == NSAppKitDefined || [event type] == NSSystemDefined)
         ok = NO;
 
-    [view unlockFocus];
+    //[view unlockFocus];
 
     if (!ok)
     {
@@ -279,7 +290,7 @@
         return NO;
     }
     dirty = YES;
-    [view cacheGraphic:self];
+    //[view cacheGraphic:self];
 
     return YES;
 }
@@ -1167,6 +1178,25 @@
     dirty = YES;
 }
 
+- (id)contourOpenLeftOrRight:(float)w
+{   VGraphic    *g = [VPath path];
+    BOOL        dir = [self isDirectionCCW];
+
+    /* we just return a copy */
+    if ( (Diff(w, 0.0) < 0.0001 && width == 0.0) )
+        return [[self copy] autorelease];
+
+    if ( w < 0.0 && !dir )
+        w = -w; // correction to left (inside) needed
+    else if ( w > 0.0 && !dir )
+        w = -w; // correction to right(outside here) needed
+
+    [self setFilled:YES];
+    g = [self contour:w];
+    [self setFilled:NO];
+    return g;
+}
+
 /*
  * return a path representing the outline of us
  * modified: 2010-03-03 (setDirectionCCW:)
