@@ -1,11 +1,12 @@
 /* Type1Font.m
  * Type 1 Font object used for the type 1 import
  *
- * Copyright (C) 1996-2014 by Cenon GmbH
+ * Copyright (C) 1996-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  2000-11-26
- * modified: 2014-07-06 ("4 random bytes" with legal characters)
+ * modified: 2015-03-13 ("-writeToFile: reduced to simplest c for subrs and charString ecryption use NSUTF8StringEncoding)
+ *           2014-07-06 ("4 random bytes" with legal characters)
  *           2012-02-07 (-writeToFile: 2x use -writeToFile:...encoding:error:)
  *
  * This program is free software; you can redistribute it and/or
@@ -57,7 +58,7 @@
 - init
 {
     gridOffset = 150.0; // 150.0;
-    afmNoPlace = [[NSArray alloc] initWithObjects:@"Aacute", @"Acircumflex", @"Adieresis", @"Agrave", @"Aring", @"Atilde", @"Ccedilla", @"Eacute", @"Ecircumflex", @"Edieresis", @"Egrave", @"Eth", @"Iacute", @"Icircumflex", @"Idieresis", @"Igrave", @"Ntilde", @"Oacute", @"Ocircumflex", @"Odieresis", @"Ograve", @"Otilde", @"Thorn", @"Uacute", @"Ucircumflex", @"Udieresis", @"Ugrave", @"Yacute", @"aacute", @"acircumflex", @"adieresis", @"agrave", @"aring", @"atilde", @"ccedilla", @"copyright", @"divide", @"eacute", @"ecircumflex", @"edieresis", @"egrave", @"eth", @"iacute", @"icircumflex", @"idieresis", @"igrave", @"logicalnot", @"mu", @"multiply", @"ntilde", @"oacute", @"ocircumflex", @"odieresis", @"ograve", @"onehalf", @"onequarter", @"onesuperior", @"otilde", @"plusminus", @"registered", @"thorn", @"threequarters", @"threesuperior", @"twosuperior", @"uacute", @"ucircumflex", @"udieresis", @"ugrave", @"yacute", @"ydieresis", nil];
+    afmNoPlace = [[NSMutableArray alloc] initWithObjects:@"Aacute", @"Acircumflex", @"Adieresis", @"Agrave", @"Aring", @"Atilde", @"Ccedilla", @"Eacute", @"Ecircumflex", @"Edieresis", @"Egrave", @"Eth", @"Iacute", @"Icircumflex", @"Idieresis", @"Igrave", @"Ntilde", @"Oacute", @"Ocircumflex", @"Odieresis", @"Ograve", @"Otilde", @"Thorn", @"Uacute", @"Ucircumflex", @"Udieresis", @"Ugrave", @"Yacute", @"aacute", @"acircumflex", @"adieresis", @"agrave", @"aring", @"atilde", @"ccedilla", @"copyright", @"divide", @"eacute", @"ecircumflex", @"edieresis", @"egrave", @"eth", @"iacute", @"icircumflex", @"idieresis", @"igrave", @"logicalnot", @"mu", @"multiply", @"ntilde", @"oacute", @"ocircumflex", @"odieresis", @"ograve", @"onehalf", @"onequarter", @"onesuperior", @"otilde", @"plusminus", @"registered", @"thorn", @"threequarters", @"threesuperior", @"twosuperior", @"uacute", @"ucircumflex", @"udieresis", @"ugrave", @"yacute", @"ydieresis", nil];
     afmNoPlaceCnt = [afmNoPlace count];
     metricsCnt = 0;
     scale = 10.0; // back scale when we save !!
@@ -879,13 +880,14 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
         return NO;
     }
     /* rename to backup */
-    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename handler:nil]) ||
-         ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename handler:nil]) )
+    if ( ([fileManager fileExistsAtPath:backupFilename] && ![fileManager removeFileAtPath:backupFilename error:nil]) ||
+         ([fileManager fileExistsAtPath:filename]       && ![fileManager movePath:filename toPath:backupFilename error:nil]) )
     {   NSRunAlertPanel(SAVE_TITLE, CANT_CREATE_BACKUP, nil, nil, nil);
         return NO;
     }
     /* create file directory */
-    else if ( isDirectory && [fileManager createDirectoryAtPath:filename recursive:NO attributes:nil error:NULL] )
+    else if ( isDirectory && [fileManager createDirectoryAtPath:filename
+                                                      recursive:NO attributes:nil error:nil] )
     {
         fileDirectory = filename;
         filename = [fileDirectory stringByAppendingPathComponent:[[filename stringByDeletingPathExtension] lastPathComponent]];
@@ -894,155 +896,255 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 
     /* save */
     if ([fileManager isWritableFileAtPath:fileDirectory])
-    {   NSString    *fontStr, *eexecStr, *afmStr;
-        int			i, len=0;
+    {   NSString    *fontStr, *afmStr; //, *eexecStr
+        NSData      *fontData = nil;
+        char        fontStrC[150001]="", eexecStrC[100000]="", str[50000]="", *exCP;
+        int			i, j, len=0, eexexLen=0;
+        const char  *fontStrConstC = NULL;
 
         fontStr = [NSString stringWithFormat:@"%%!FontType1-1.0: %@ %@\n", fontName, fontInfo.version];
 
         fontStr = [fontStr stringByAppendingFormat:@"%%%%CreationDate: %@\n", [[NSCalendarDate date] descriptionWithCalendarFormat:@"%a %b %d %H:%M:%S %Y"]];
-	fontStr = [fontStr stringByAppendingFormat:@"%%%%VMusage: %d %d\n", 27647, 34029];
-	fontStr = [fontStr stringByAppendingFormat:@"%%%@\n", fontInfo.copyright];
 
-	/* font dict */
-	fontStr = [fontStr stringByAppendingFormat:@"14 dict begin\n"];
-	/* fontInfo dict */
-	fontStr = [fontStr stringByAppendingFormat:@"/FontInfo 10 dict dup begin\n"];
-	fontStr = [fontStr stringByAppendingFormat:@"/version (%@) readonly def\n", fontInfo.version];
-	fontStr = [fontStr stringByAppendingFormat:@"/Notice (%s) readonly def\n", COPYRIGHT];
-	fontStr = [fontStr stringByAppendingFormat:@"/FullName (%@) readonly def\n", fontInfo.fullName];
-	fontStr = [fontStr stringByAppendingFormat:@"/FamilyName (%@) readonly def\n", fontInfo.familyName];
-	fontStr = [fontStr stringByAppendingFormat:@"/Weight (%@) readonly def\n", fontInfo.weight];
-	fontStr = [fontStr stringByAppendingFormat:@"/ItalicAngle %d def\n", 0];
-	fontStr = [fontStr stringByAppendingFormat:@"/isFixedPitch %s def\n", (fontInfo.isFixedPitch)? "true" : "false"];
-	fontStr = [fontStr stringByAppendingFormat:@"/UnderlinePosition %ld def\n", fontInfo.underlinePosition];
-	fontStr = [fontStr stringByAppendingFormat:@"/UnderlineThickness %ld def\n", fontInfo.underlineThickness];
-	fontStr = [fontStr stringByAppendingFormat:@"end readonly def\n"];
-	/* font dict */
-	fontStr = [fontStr stringByAppendingFormat:@"/FontName /%@ def\n", fontName];
-	fontStr = [fontStr stringByAppendingFormat:@"/PaintType %d def\n", paintType];
-	if (paintType == 2)
-		fontStr = [fontStr stringByAppendingFormat:@"/StrokeWidth %d def\n", 10];
-	fontStr = [fontStr stringByAppendingFormat:@"/FontType %d def\n", fontType]; // 1 for type1 fonts
-//	m = fontMatrix;
-	fontStr = [fontStr stringByAppendingFormat:@"/FontMatrix [0.001 0 0 0.001 0 0] readonly def\n"];
-//	NXPrintf(stream, "/FontMatrix [%f %f %f %f %f %f] readonly def\n", m[0], m[1], m[2], m[3], m[4], m[5]);
+        fontStr = [fontStr stringByAppendingFormat:@"%%%%VMusage: %d %d\n", 27647, 34029];
+        fontStr = [fontStr stringByAppendingFormat:@"%%%@\n", fontInfo.copyright];
 
-// Encoding !!!
-	fontStr = [fontStr stringByAppendingFormat:@"/Encoding StandardEncoding def\n"];
-	fontStr = [fontStr stringByAppendingFormat:@"/FontBBox {%.0f %.0f %.0f %.0f} readonly def\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
-	fontStr = [fontStr stringByAppendingFormat:@"/UniqueID %ld def\n", uniqueID];
-	fontStr = [fontStr stringByAppendingFormat:@"currentdict end\n"];
+        /* font dict */
+        fontStr = [fontStr stringByAppendingFormat:@"14 dict begin\n"];
+        /* fontInfo dict */
+        fontStr = [fontStr stringByAppendingFormat:@"/FontInfo 10 dict dup begin\n"];
+        fontStr = [fontStr stringByAppendingFormat:@"/version (%@) readonly def\n", fontInfo.version];
+        fontStr = [fontStr stringByAppendingFormat:@"/Notice (%s) readonly def\n", COPYRIGHT];
+        fontStr = [fontStr stringByAppendingFormat:@"/FullName (%@) readonly def\n", fontInfo.fullName];
+        fontStr = [fontStr stringByAppendingFormat:@"/FamilyName (%@) readonly def\n", fontInfo.familyName];
+        fontStr = [fontStr stringByAppendingFormat:@"/Weight (%@) readonly def\n", fontInfo.weight];
+        fontStr = [fontStr stringByAppendingFormat:@"/ItalicAngle %d def\n", 0];
+        fontStr = [fontStr stringByAppendingFormat:@"/isFixedPitch %s def\n", (fontInfo.isFixedPitch)? "true" : "false"];
+        fontStr = [fontStr stringByAppendingFormat:@"/UnderlinePosition %ld def\n", fontInfo.underlinePosition];
+        fontStr = [fontStr stringByAppendingFormat:@"/UnderlineThickness %ld def\n", fontInfo.underlineThickness];
+        fontStr = [fontStr stringByAppendingFormat:@"end readonly def\n"];
+        /* font dict */
+        fontStr = [fontStr stringByAppendingFormat:@"/FontName /%@ def\n", fontName];
+        fontStr = [fontStr stringByAppendingFormat:@"/PaintType %d def\n", paintType];
+        if (paintType == 2)
+            fontStr = [fontStr stringByAppendingFormat:@"/StrokeWidth %d def\n", 10];
+        fontStr = [fontStr stringByAppendingFormat:@"/FontType %d def\n", fontType]; // 1 for type1 fonts
+    //	m = fontMatrix;
+        fontStr = [fontStr stringByAppendingFormat:@"/FontMatrix [0.001 0 0 0.001 0 0] readonly def\n"];
+    //	NXPrintf(stream, "/FontMatrix [%f %f %f %f %f %f] readonly def\n", m[0], m[1], m[2], m[3], m[4], m[5]);
 
-	/* eexec */
-	fontStr = [fontStr stringByAppendingFormat:@"currentfile eexec\n"];
+    // Encoding !!!
+        fontStr = [fontStr stringByAppendingFormat:@"/Encoding StandardEncoding def\n"];
+        fontStr = [fontStr stringByAppendingFormat:@"/FontBBox {%.0f %.0f %.0f %.0f} readonly def\n", fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]];
+        fontStr = [fontStr stringByAppendingFormat:@"/UniqueID %ld def\n", uniqueID];
+        fontStr = [fontStr stringByAppendingFormat:@"currentdict end\n"];
 
-        eexecStr = [NSString stringWithFormat:@"%s", "]|]|"]; /* 4 random bytes */
-	eexecStr = [eexecStr stringByAppendingFormat:@"userdict"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/RD{string currentfile exch readstring pop}executeonly put\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"userdict/ND{noaccess def}executeonly put\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"userdict/NP{noaccess put}executeonly put\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"dup/Private 8 dict dup begin\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/BlueValues [%.0f %.0f %.0f %.0f %.0f %.0f] noaccess def\n", privateDict.blueValues[0], privateDict.blueValues[1], privateDict.blueValues[2], privateDict.blueValues[3], privateDict.blueValues[4], privateDict.blueValues[5]];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/MinFeature{%.0f %.0f}noaccess def\n", privateDict.minFeature[0], privateDict.minFeature[1]];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/password 5839 def\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/Source (%@) readonly def\n", privateDict.source];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/UniqueID %ld def\n", uniqueID];
-	eexecStr = [eexecStr stringByAppendingFormat:@"/OtherSubrs[{}{}{}\n\
-  {  %% coloring subr number on stack\
-     systemdict /internaldict known not\n\
-       {pop 3} %% return null subr\n\
-       { 1183615869 systemdict /internaldict get exec\n\
-         dup /startlock known\n\
-           {/startlock get exec} %% return new coloring subr\n\
-           { dup /strtlck known %% other name for startlock\n\
-	      {/strtlck get exec}\n\
-              {pop 3} %% return null subr\n\
-	      ifelse}\n\
-	 ifelse }\n\
-       ifelse\n\
-  } executeonly\n\
-]noaccess def\n"];
+        /* eexec */
+        fontStr = [fontStr stringByAppendingFormat:@"currentfile eexec\n"];
 
-	/* subrs */
-        eexecStr = [eexecStr stringByAppendingFormat:@"/Subrs %d array\n", privateDict.subrsCnt];
 
-// done in [fontObhect update]
-#if 0
-	/* subrs */
-	for (i=0; i<privateDict.subrsCnt; i++)
-	{   unsigned char	*up = subrs[i].proc;
-            subrs[i].proc = malloc(subrs[i].length+5);
-            memcpy(subrs[i].proc+4, up, subrs[i].length);
-            subrs[i].length += 4;
-            free(up);
-            subrs[i].proc = encryptCharString(subrs[i].proc, subrs[i].length);
-	}
-#endif
-	//for (i=0; i<subrsCnt; i++)
-	for (i=0; i<privateDict.subrsCnt; i++)
-	{   NSString	*str = [NSString stringWithCString:(char*)privateDict.subrs[i].proc
-                                                length:privateDict.subrs[i].length];
 
-            eexecStr = [eexecStr stringByAppendingFormat:@"dup %d %d RD ", i, privateDict.subrs[i].length];
-            eexecStr = [eexecStr stringByAppendingString:str];
-            eexecStr = [eexecStr stringByAppendingFormat:@" NP\n"];
-	}
-	eexecStr = [eexecStr stringByAppendingFormat:@"noaccess def\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"end noaccess put\n"];
-
-	/* char strings */
-	eexecStr = [eexecStr stringByAppendingFormat:@"dup /CharStrings %d dict dup begin\n", charStringCnt];
-
-// done in [fontObhect update]
-#if 0
-	/* char strings */
-	for (i=0; i<charStringCnt; i++)
-        {   unsigned char	*up = charStrings[i].code;
-            charStrings[i].code = malloc(charStrings[i].length+5);
-            memcpy(charStrings[i].code+4, up, charStrings[i].length);
-            charStrings[i].length += 4;
-            free(up);
-            charStrings[i].code = encryptCharString(charStrings[i].code, charStrings[i].length);
+        if ( [fontStr canBeConvertedToEncoding:NSUTF8StringEncoding] ) // NSASCIIStringEncoding
+            fontStrConstC = [fontStr cStringUsingEncoding:NSUTF8StringEncoding];
+        else
+        {   NSLog(@"Cant save fontStr to Ascii");
+            return NO;
         }
-#endif
-	for (i=0; i<charStringCnt; i++)	/* georg */
-	{   NSString    *str=[NSString stringWithCString:(char*)charStrings[i].code
-                                              length:charStrings[i].length];
+        len = strlen(fontStrConstC);
+        strncpy(fontStrC, (char*)fontStrConstC, len);
+        fontStrC[len] = 0;
 
-            eexecStr = [eexecStr stringByAppendingFormat:@"/%s %d RD ", charStrings[i].name, charStrings[i].length];
-            eexecStr = [eexecStr stringByAppendingString:str];
-            eexecStr = [eexecStr stringByAppendingFormat:@" ND\n"];
-	}
-	eexecStr = [eexecStr stringByAppendingFormat:@"end\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"readonly put\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"dup/FontName get exch definefont pop\n"];
-	eexecStr = [eexecStr stringByAppendingFormat:@"mark currentfile closefile\n"];
-#   if 0
-        { NSString	*file = [NSString stringWithFormat:@"/Net/nesquick/Users/ilonka/Tempo/Test/testSubrs"];
-            [eexecStr writeToFile:file atomically:YES];
+        strcat(eexecStrC, "]|]|"); // 4 random bytes
+        strcat(eexecStrC, "userdict");
+            strcat(eexecStrC, "/RD{string currentfile exch readstring pop}executeonly put\n");
+            strcat(eexecStrC, "userdict/ND{noaccess def}executeonly put\n");
+            strcat(eexecStrC, "userdict/NP{noaccess put}executeonly put\n");
+            strcat(eexecStrC, "dup/Private 8 dict dup begin\n");
+
+
+        len = sprintf(str, "/BlueValues [%.0f %.0f %.0f %.0f %.0f %.0f] noaccess def\n", privateDict.blueValues[0], privateDict.blueValues[1], privateDict.blueValues[2], privateDict.blueValues[3], privateDict.blueValues[4], privateDict.blueValues[5]);
+        strcat(eexecStrC, str);
+
+        len = sprintf(str, "/MinFeature{%.0f %.0f}noaccess def\n",privateDict.minFeature[0], privateDict.minFeature[1]);
+        strcat(eexecStrC, str);
+
+        strcat(eexecStrC, "/password 5839 def\n");
+
+        len = sprintf(str, "/Source (%s) readonly def\n",[privateDict.source cStringUsingEncoding:NSUTF8StringEncoding]);
+        strcat(eexecStrC, str);
+        len = sprintf(str, "/UniqueID %d def\n", (int)uniqueID);
+        strcat(eexecStrC, str);
+
+        strcat(eexecStrC, "/OtherSubrs[{}{}{}\n\
+          {  %% coloring subr number on stack\
+             systemdict /internaldict known not\n\
+               {pop 3} %% return null subr\n\
+               { 1183615869 systemdict /internaldict get exec\n\
+                 dup /startlock known\n\
+                   {/startlock get exec} %% return new coloring subr\n\
+                   { dup /strtlck known %% other name for startlock\n\
+                  {/strtlck get exec}\n\
+                      {pop 3} %% return null subr\n\
+                  ifelse}\n\
+             ifelse }\n\
+               ifelse\n\
+          } executeonly\n\
+        ]noaccess def\n");
+
+        len = sprintf(str, "/Subrs %d array\n", (int)privateDict.subrsCnt);
+        strcat(eexecStrC, str);
+
+        exCP = eexecStrC;
+        eexexLen=strlen(eexecStrC);
+        exCP += eexexLen;
+
+        /* some privateDict.subrs[i].proc and charStrings[i].code
+         * have 0 inside after encrypt, so we cant work with any string function or method any more
+         */
+
+        for (i=0; i<privateDict.subrsCnt; i++)
+        {
+/*
+char    *subrsProcChar = malloc((privateDict.subrs[i].length+1)*sizeof(UBYTE));
+int     rLen;
+
+memcpy(subrsProcChar, privateDict.subrs[i].proc, privateDict.subrs[i].length);
+
+if ( (rLen=strlen(subrsProcChar)) < privateDict.subrs[i].length)
+    NSLog(@"Type1Font.m: -writeToFile: subrs proc string 0 inside was denn nun tun");
+
+free(subrsProcChar);
+*/
+
+            //strcat(eexecStrC, str);
+            len = sprintf(str, "dup %d %d RD ", i, privateDict.subrs[i].length);
+            for (j=0; j<len; j++)
+            {
+                *exCP = str[j];
+                exCP++;
+            }
+            eexexLen += len;
+
+            //strncat(eexecStrC, (char*)privateDict.subrs[i].proc, privateDict.subrs[i].length);
+            for (j=0; j<privateDict.subrs[i].length; j++)
+            {
+                *exCP = (privateDict.subrs[i].proc)[j];
+                exCP++;
+            }
+            eexexLen += privateDict.subrs[i].length;
+
+            //strcat(eexecStrC, " NP\n");
+            len = sprintf(str, " NP\n");
+            for (j=0; j<len; j++)
+            {
+                *exCP = str[j];
+                exCP++;
+            }
+            eexexLen += len;
         }
-#   endif
+
+        //strcat(eexecStrC, "noaccess def\nend noaccess put\n");
+        //strcat(eexecStrC, "end noaccess put\n");
+        len = sprintf(str, "noaccess def\nend noaccess put\n");
+        for (j=0; j<len; j++)
+        {
+            *exCP = str[j];
+            exCP++;
+        }
+        eexexLen += len;
+
+        len = sprintf(str, "dup /CharStrings %d dict dup begin\n", charStringCnt);
+        //strcat(eexecStrC, str);
+        for (j=0; j<len; j++)
+        {
+            *exCP = str[j];
+            exCP++;
+        }
+        eexexLen += len;
+
+        for (i=0; i<charStringCnt; i++)	/* georg */
+        {
+/*
+char    *charStr = malloc((charStrings[i].length+1)*sizeof(UBYTE));
+int rLen;
+
+    memcpy(charStr, charStrings[i].code, charStrings[i].length);
+
+if ( (rLen=strlen(charStr)) < charStrings[i].length)
+    NSLog(@"Type1Font.m: -writeToFile: charStr code string 0 inside was denn nun tun");
+        free(charStr);
+*/
+
+            len = sprintf(str, "/%s %d RD ", charStrings[i].name, charStrings[i].length);
+            for (j=0; j<len; j++)
+            {
+                *exCP = str[j];
+                exCP++;
+            }
+            eexexLen += len;
+
+            //strcat(eexecStrC, (char*)charStr); // (char*)charStrings[i].code)
+            for (j=0; j<charStrings[i].length; j++)
+            {
+                *exCP = (charStrings[i].code)[j];
+                exCP++;
+            }
+            eexexLen += charStrings[i].length;
+
+
+            //strcat(eexecStrC, " NP\n");
+            len = sprintf(str, " NP\n");
+            for (j=0; j<len; j++)
+            {
+                *exCP = str[j];
+                exCP++;
+            }
+            eexexLen += len;
+        }
+
+        /*
+        strcat(eexecStrC, "end\n");
+        strcat(eexecStrC, "readonly put\n");
+        strcat(eexecStrC, "dup/FontName get exch definefont pop\n");
+        strcat(eexecStrC, "mark currentfile closefile\n");
+        */
+        len = sprintf(str, "end\nreadonly put\ndup/FontName get exch definefont pop\nmark currentfile closefile\n");
+        for (j=0; j<len; j++)
+        {
+            *exCP = str[j];
+            exCP++;
+        }
+        eexexLen += len;
+
         // NXGetMemoryBuffer(eexecStream, &buf, &len, &maxlen);
-        {   char	*eExecCStr, *buffer;
+        {   char	*eExecCStr;
 
-            len = [eexecStr length]; // +1 -> 0
-            buffer = malloc(len+1);
-            [eexecStr getCString:buffer];   // FIXME: use UTF8String and make a copy
-            eExecCStr = (char*)encryptEexec((unsigned char*)buffer, len);
-            fontStr = [fontStr stringByAppendingString:[NSString stringWithCString:eExecCStr length:len*2]];
+            eExecCStr = (char*)encryptEexec((unsigned char*)eexecStrC, eexexLen);
+
+//if (strlen(eExecCStr) != eexexLen/2)
+//                NSLog(@"Type1Font.m: -writeToFile: eExecCStr zu kurz ?");
+
+            strcat(fontStrC, eExecCStr);
             free(eExecCStr);
-            free(buffer);
         }
-        // NXWrite(stream, encryptEexec(buf, len), len*2);
 
-        fontStr = [fontStr stringByAppendingFormat:@"\n"];
-        for (i=0;i<512;i++)
-            fontStr = [fontStr stringByAppendingFormat:@"0"];
-        fontStr = [fontStr stringByAppendingFormat:@"\ncleartomark"];
+        len = strlen(fontStrC);
+        fontData = [NSData dataWithBytes:fontStrC length:len];
 
-        //savedOk = [fontStr writeToFile:filename atomically:YES];
-        savedOk = [fontStr writeToFile:filename atomically:YES
-                              encoding:NSUTF8StringEncoding error:NULL];    // >= 10.5
+            //savedOk = [fontStr writeToFile:filename atomically:YES];
+            //savedOk = [fontStr writeToFile:filename atomically:YES
+            //                      encoding:NSUTF8StringEncoding error:NULL];    // >= 10.5 //  NSString did not work with 0 inside
+
+        savedOk = [fontData writeToFile:filename atomically:YES];    // >= 10.5
+
+        /* restore backup */
+        if (!savedOk)
+        {
+            [fileManager removeFileAtPath:fileDirectory error:nil];	// remove what we just started to write
+            [fileManager movePath:backupFilename toPath:fileDirectory error:nil];	// restore backup
+            NSRunAlertPanel(SAVE_TITLE, CANT_SAVE, nil, nil, nil);
+            return YES;
+        }
 
         afmStr = [NSString stringWithFormat:@"StartFontMetrics 2.0\n"];
         afmStr = [afmStr stringByAppendingFormat:@"Comment %s\n", COPYRIGHT];
@@ -1075,7 +1177,7 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
 
         //savedOk = [afmStr writeToFile:afmfilename atomically:YES];
         savedOk = [afmStr writeToFile:afmfilename atomically:YES
-                             encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5
+                             encoding:NSUTF8StringEncoding error:NULL]; // >= 10.5 // NSUTF8StringEncoding NSASCIIStringEncoding
     }
     else
         NSRunAlertPanel(SAVE_TITLE, DIR_NOT_WRITABLE, nil, nil, nil);
@@ -1083,14 +1185,14 @@ int getCharFromList(id self, NSArray *alist, FontState *fontState, unsigned char
     /* restore backup */
     if (!savedOk)
     {
-        [fileManager removeFileAtPath:fileDirectory handler:nil];	// remove what we just started to write
-        [fileManager movePath:backupFilename toPath:fileDirectory handler:nil];	// restore backup
+        [fileManager removeFileAtPath:fileDirectory error:nil];	// remove what we just started to write
+        [fileManager movePath:backupFilename toPath:fileDirectory error:nil];	// restore backup
         NSRunAlertPanel(SAVE_TITLE, CANT_SAVE, nil, nil, nil);
     }
     else
     {
         if (Prefs_RemoveBackups)
-            [fileManager removeFileAtPath:backupFilename handler:nil];
+            [fileManager removeFileAtPath:backupFilename error:nil];
     }
     return YES;
 }
