@@ -1,11 +1,12 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2014 by Cenon GmbH
+ * Copyright (C) 1996-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2014-08-29 (-getArc:::: if group.ext transform the complete arc)
+ * modified: 2015-06-19 (-getText: remove formatting codes from string, log added
+ *           2014-08-29 (-getArc:::: if group.ext transform the complete arc)
  *           2014-08-29 (-scanGroup: group.color no init)
  *           2014-07-04 (-getText: fontName added to internal declaration)
  *           2013-09-13 (getInsert(), ...: place objects to layer of insert, not the layer of the blocks)
@@ -22,7 +23,7 @@
  *           2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
  *           2007-01-30 (close PolyLine: check for start == end point)
  *
- * This file is part of the vhf Import Library.
+ * This file is part of the Cenon Import Library.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the Cenon Public License as
@@ -74,6 +75,7 @@ static BOOL gotoGroup( NSScanner *scanner, NSString *name);
 static NSDictionary *getTableFromData(NSString *data);
 static NSColor *getColorFromLayer(NSString *str, NSArray *table);
 static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle);
+static V3Point transformOCS(V3Point pOCS, V3Point AZ);
 
 @interface DXFImport(PrivateMethods)
 - (BOOL)getHeader:(NSString*)data;
@@ -325,26 +327,26 @@ static NSDictionary *getTableFromData(NSString *data)
         gotoNextLine(scanner);	// goto value line
         switch ( ident )
         {
-            case ID_GROUP:
+            case ID_GROUP:      // (0)
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     string = @"";
                 if (![string isEqual:GRPSTYLE]) // ENDTAB
                     break;
                 dict = [NSMutableDictionary dictionary];
                 break;
-            case ID_NAME:
+            case ID_NAME:       // (2)
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     break;
                 [dict setObject:string forKey:@"name"];
                 [styles setObject:dict forKey:string];
                 //[styles addObject:dict];
                 break;
-            case ID_DESCRIPT:   // font file name ("ariblk.TTF")
+            case ID_DESCRIPT:   // (3)    font file name ("ariblk.TTF")
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     break;
                 [dict setObject:string forKey:@"fileName"];
                 break;
-            case ID_EXTSTRING:  // font name ("Arial Black")
+            case ID_EXTSTRING:  // (1000) font name ("Arial Black")
                 if (![scanner scanUpToCharactersFromSet:newLineSet intoString:&string])
                     break;
                 [dict setObject:string forKey:@"fontName"];
@@ -822,8 +824,8 @@ static NSColor *getColorFromLayer(NSString *str, NSArray *layers)
 }
 
 
-/* created:   17.05.93
- * modified:  18.05.93 10.03.97
+/* created:   1993-05-17
+ * modified:  2015-10-08
  *
  * purpose:   get line
  * parameter: scanner (the dxf data)
@@ -1173,7 +1175,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
         return;
     }
     h = (a*l)/2.0;                      // height of arc
-    mx = (begx+endx)/2.0; my = (begy+endy)/2.0; // mit point between start and end
+    mx = (begx+endx)/2.0; my = (begy+endy)/2.0; // mid point between start and end
     dx = mx - begx;                     // 
     dy = my - begy;                     // 
     //if (h>0)	/* don't know if this is correct */
@@ -1286,7 +1288,8 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 }
 
 /* created:   1993-05-17
- * modified:  2014-07-23 (if group.ext transform the complete arc)
+ * modified:  2015-10-08 (with extrusion angle and equal angle, invert angle)
+ *            2014-07-23 (if group.ext transform the complete arc)
  *            2010-04-03 (negative 360 deg arcs work now)
  *
  * purpose:   get arc
@@ -1296,7 +1299,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
  *            angle
  */
 - (BOOL)getArc:(NSScanner*)scanner :(NSPoint*)ctr :(NSPoint*)start :(float*)angle
-{   float	r, ba, ea;
+{   VFloat	r, ba, ea;
     NSRect	rect;
 
     [self scanGroup:scanner];
@@ -1304,19 +1307,21 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
         return NO;
 
     /* revert transformation of group.x0 group.y0 point - we need to transform the complete arc */
-    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
-    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+    /*if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
+    {   V3Point N = V3MakePoint(-group.extX, -group.extY, -group.extZ), p;  // FIXME: not like this
 
-        p = transformOCS(V3MakePoint(group.x0, group.y0, 0.0), N);
-        group.x0 = p.x; group.y0 = p.y;
-    }
+        p = transformOCS(V3MakePoint(group.x0, group.y0, group.z0), N);
+        group.x0 = p.x; group.y0 = p.y; group.z0 = p.z;
+    }*/
 
     state.color = [self currentColor];
-    ctr->x = DeviceResToInternal(group.x0, res) + state.offset.x;
-    ctr->y = DeviceResToInternal(group.y0, res) + state.offset.y;
-    //FIXME: group.extX, group.extY, group.extZ
-    r = DeviceResToInternal(group.width, res);
-    ba = group.begAngle;
+    ctr->x = group.p0.x;
+    ctr->y = group.p0.y;
+    r      = group.width;
+    //ctr->x = DeviceResToInternal(group.x0, res) + state.offset.x; // at end !
+    //ctr->y = DeviceResToInternal(group.y0, res) + state.offset.y;
+    //r      = DeviceResToInternal(group.width, res);
+    ba = group.begAngle;    // arcs are ccw (except extZ=-1, or screwed up coordinate system (extrusion direction)
     ea = group.endAngle;
 
     start->x = ctr->x + r;
@@ -1330,29 +1335,46 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 
     if ( *angle == 0.0 )
         *angle = ea - ba;
+    if ( *angle == 0.0 )    // angle == 0 => 360
+        *angle = 360.0;
 
-    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
-    {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
+    if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )  // see: ArbAxis-Conversion_1.rtf
+    {   V3Point N   = V3MakePoint(group.extX, group.extY, group.extZ), p;
         NSPoint end = vhfPointAngleFromRefPoint(*ctr, *start, *angle);
 
-        p = transformOCS(V3MakePoint(ctr->x, ctr->y, 0.0), N);
-        ctr->x = p.x; ctr->y = p.y;
-        p = transformOCS(V3MakePoint(start->x, start->y, 0.0), N);
+        p = transformOCS(V3MakePoint(ctr->x,   ctr->y,   group.p0.z), N);
+        ctr->x   = p.x; ctr->y   = p.y; // ctr->x = group.x0; ctr->y = group.y0;
+        p = transformOCS(V3MakePoint(start->x, start->y, group.p0.z), N);
         start->x = p.x; start->y = p.y;
-        p = transformOCS(V3MakePoint(end.x, end.y, 0.0), N);
-        end.x = p.x; end.y = p.y;
+        p = transformOCS(V3MakePoint(end.x,    end.y,    group.p0.z), N);
+        end.x    = p.x; end.y    = p.y;
 
+        /* standard direction in DXF is ccw
+         * 1. extZ = -1 inverts the direction
+         * 2. a transformed coordinate system gives us no idea in what direction the angle goes, we leave it ccw.
+         *    FIXME: a possibility would be tracking the last point, or if unsorted connecting to elements
+         */
         ba = vhfAngleOfPointRelativeCenter(*start, *ctr);
-        ea = vhfAngleOfPointRelativeCenter(end, *ctr);
-        if ( Abs(ea-ba) == Abs(*angle) )
-            *angle = ea - ba;
-        else
-        {   if ( ba < ea )
-            ba += 360.0;
+        ea = vhfAngleOfPointRelativeCenter(end,    *ctr);
+        if ( group.extX == 0.0 && group.extY == 0.0 && group.extZ == -1.0 ) // 1. inverted direction ccw -> cw
+            *angle = -*angle;
+        else if ( Abs(ea-ba) == Abs(*angle) )
+            //*angle = ea - ba;
+            *angle = -*angle;                                               // 2015-10-08 ???
+        else                                                                // 2. translated coordinate system
+        {   //if ( ba < ea )
+            //    ba += 360.0;
+            if (ea < ba)
+                ea += 360.0;
             *angle = ea - ba;
         }
     }
 
+    ctr->x   = DeviceResToInternal(ctr->x,   res) + state.offset.x;
+    ctr->y   = DeviceResToInternal(ctr->y,   res) + state.offset.y;
+    start->x = DeviceResToInternal(start->x, res) + state.offset.x;
+    start->y = DeviceResToInternal(start->y, res) + state.offset.y;
+
     state.width = 0.0;
 
     //[self updateBounds:*start];
@@ -1366,7 +1388,8 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
 }
 
 /* created:   1993-06-11
- * modified:  2013-04-16 (fontName added)
+ * modified:  2015-06-19 (text formatting: remove formatting codes, log added)
+ *            2013-04-16 (fontName added)
  *
  * purpose:   get text (TEXT, MTEXT)
  *            read a sv-polygon for each character
@@ -1400,6 +1423,91 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
     state.color = [self currentColor];
     textHeight = *size = DeviceResToInternal(group.width, res) * 3.0/2.0;
     *string = [[group.text stringByReplacing:@"\\P" by:@"\n"] retain];	// convert newline
+    /* text formatting: "{\fDINMittelschrift|b1||i0||c0|p34;\C7;\H3.;\W1.;Versuchstext}"
+     * TODO: turn formatting codes into attributed string (NSAttributedString)
+     */
+    if ( [*string rangeOfString:@"{"].length )
+    {   NSMutableString *mString = [*string mutableCopy];
+        NSRange         range, searchRange = NSMakeRange(0, 0);
+        unichar         uchar;
+        int             i;
+
+        /* remove "{", "}", but not "\{", "\}" */
+        range = [mString rangeOfString:@"{"];   // {...
+        if ( range.length && (range.location == 0 || [mString characterAtIndex:range.location-1] != '\\') )
+            [mString replaceCharactersInRange:range withString:@""];
+        range = [mString rangeOfString:@"}"];   // ...}
+        if ( range.length && (range.location == 0 || [mString characterAtIndex:range.location-1] != '\\') )
+            [mString replaceCharactersInRange:range withString:@""];
+
+        /* remove "\[a-zA-Z]...;" */
+        for ( i=0; i<100; i++ ) // up to 100 replacements, just to be safe
+        {   NSRange range1;
+
+            searchRange.length = [mString length] - searchRange.location;
+            range = [mString rangeOfString:@"\\" options:0 range:searchRange];          // "\"
+            if ( range.length == 0 )
+                break;
+            uchar = [mString characterAtIndex:range.location+1];
+            searchRange = NSMakeRange(range.location, [mString length] - range.location);
+            switch ( uchar )
+            {
+                case 'a':   // \A0;     alignment
+                case 'A':
+                case 'c':   // \C1;     color change
+                case 'C':
+                case 'f':   // \F...;   font selection
+                case 'F':
+                case 'h':   // \H3x;    text height
+                case 'H':
+                case 'q':   // \Q30;    slanting
+                case 'Q':
+                case 't':   // \T2;     tracking, char spacing
+                case 'T':
+                case 'w':   // \W0.8x;  text width
+                case 'W':
+                    range1 = [mString rangeOfString:@";" options:0 range:searchRange];   // "\...;"
+                    if ( range1.length )
+                    {   range.length = range1.location+1;
+                        // TODO: interpret into attributed string
+                        [mString replaceCharactersInRange:range withString:@""];
+                    }
+                    else
+                        NSLog(@"DXF-Import, MTEXT: missing \";\" for code '%c' in string: \"%@\"", uchar, mString);
+                    searchRange.location = 0;
+                    break;
+                case 'K':   // \K       start strike-through
+                case 'k':   // \k       stop  strike-through
+                case 'L':   // \L       start underline
+                case 'l':   // \l       stop  underline
+                case 'O':   // \O       start overstrike
+                case 'o':   // \o       stop  overstrike
+                case 'X':   // \X       Paragraph wrap
+                    range.length = 2;
+                    // TODO: interpret into attributed string
+                    [mString replaceCharactersInRange:range withString:@""];
+                    searchRange.location = 0;
+                    break;
+                case 'P':   // \P       new line
+                    range.length = 2;
+                    [mString replaceCharactersInRange:range withString:@"\n"];
+                    searchRange.location = 0;
+                    break;
+                case '\\':
+                case '{':
+                case '}':
+                    searchRange.location ++;
+                    break;
+                default:    // skip: "\\", "\{", "\}"
+                    searchRange.location ++;
+                    break;
+            }
+        }
+
+        NSLog(@"DXF-Import, MTEXT: text formatting ignored: \"%@\"", *string);
+        [*string release];
+        *string = mString;
+    }
     origin->x = DeviceResToInternal(group.x0, res) + state.offset.x;
     origin->y = DeviceResToInternal(group.y0, res) + state.offset.y;
     *angle = group.begAngle;
@@ -1420,7 +1528,7 @@ static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *an
         else if ( [style objectForKey:@"fileName"] )
         {   NSString    *fn = [style objectForKey:@"fileName"];
 
-            NSLog(@"TODO: what are we going to assign to the fonbt file name: \"%@\" ? Using Default.", fn);
+            NSLog(@"DXF-Import, MTEXT: unknown font file: \"%@\". Using Default.", fn);
             *fontName = nil;
         }
     }
@@ -1675,7 +1783,7 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
     AY = crossProduct(AZ, AX);
     AY = scaleToUnit( AY );
 
-    /* these are direction cosines, we have to rotate each axis around this */
+    /* AX, AY, AZ are direction cosines, we have to rotate each axis around this */
     {   double x = pOCS.x, y = pOCS.y, z = pOCS.z;
 
         p.x = x * AX.x + y * AY.x + z * AZ.x;
@@ -1700,10 +1808,10 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
     /* FIXME: default values may overwrite value reached down from hierarchy !
      * first/specialy color and layer - even layer should always be set
      */
-    [group.text release];   group.text = nil;
-    [group.style release];  group.style = nil;
-    [group.name release];   group.name = nil;
-    [group.layer release];  group.layer = nil;
+    [group.text   release]; group.text   = nil;
+    [group.style  release]; group.style  = nil;
+    [group.name   release]; group.name   = nil;
+    [group.layer  release]; group.layer  = nil;
     [group.handle release]; group.handle = nil;
     // group.color = 0; // we use the color from higher Element, if color is not set
     group.width    = -1.0;
@@ -1866,8 +1974,9 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
             case ID_EXT_Y:
                 if (![scanner scanFloat:&group.extY])
                     group.extY = 0.0;
-                if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction
-                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %u", (unsigned)[scanner scanLocation]);
+                if ( group.extX != 0.0 || group.extY != 0.0 )   // FIXME: extrusion direction (should be supported, but working ?)
+                    NSLog(@"DXF-Import: extrusion direction not supported. File-Location = %u", location);
+                          //[[scanner string] substringWithRange:NSMakeRange(Max(location-20, 0), 40)]
                 break;
             case ID_EXT_Z:
                 if (![scanner scanFloat:&group.extZ])
@@ -1886,6 +1995,7 @@ static V3Point transformOCS(V3Point pOCS, V3Point AZ)
     }
 
     /* transform coordinates from OCS to WCS (only if extrusion direction tells us so) */
+    group.p0.x = group.x0; group.p0.y = group.y0; group.p0.z = group.z0;    // hack for Arc
     if ( group.extX != 0.0 || group.extY != 0.0 || group.extZ != 1.0 )
     {   V3Point N = V3MakePoint(group.extX, group.extY, group.extZ), p;
 
