@@ -1,11 +1,12 @@
 /* dvUndo.h
  * Undo additions for Cenon DocView class
  *
- * Copyright (C) 1999-2014 by Cenon GmbH
+ * Copyright (C) 1999-2015 by Cenon GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1999-
- * modified: 2013-01-15 (-rotate:, rotate around coordBounds of objects)
+ * modified: 2015-07-30 (-splitObject:, -addPointTo: [self setNeedsDisplayInRect:] (not -drawRect:))
+ *           2013-01-15 (-rotate:, rotate around coordBounds of objects)
  *           2009-03-06 (-rotate: -deselectLockedLayers:..)
  *           2005-07-28 (-addPointTo: added)
  *
@@ -24,7 +25,7 @@
  * with this program; see the file LICENSE. If not, write to Cenon.
  *
  * Cenon GmbH, Schwabstr. 45, 72108 Rottenburg a.N., Germany
- * eMail: info@vhf.biz
+ * eMail: info@cenon.de
  * http://www.cenon.info
  */
 
@@ -539,7 +540,7 @@
 
 /* split object at point
  * created:  2002-11-27
- * modified: 
+ * modified: 2015-07-30 ([self setNeedsDisplayInRect:] (not -drawRect:))
  *
  * get objects splitted at point
  * save changes for undo
@@ -548,11 +549,11 @@
  * redraw affected area (selection only)
  */
 - (void)splitObject:(VGraphic*)g atPoint:(NSPoint)p redraw:(BOOL)redraw
-{   id			change;
-    int			l, cnt = [slayList count];
-    NSRect		affectedBounds = [g extendedBoundsWithScale:scale]; // object bounds
-    NSMutableArray	*nglist=nil;
-    BOOL		gIsFilled = [g filled];
+{   id              change;
+    int             l, cnt = [slayList count];
+    NSRect          affectedBounds = [g extendedBoundsWithScale:scale]; // object bounds
+    NSMutableArray  *nglist = nil;
+    BOOL            gIsFilled = [g filled];
 
     change = [[ContourGraphicsChange alloc] initGraphicView:self];
     [change startChange];
@@ -565,7 +566,7 @@
                 continue;
             if ([[layer list] indexOfObject:g] != NSNotFound) // object in this layer
             {
-                if ((nglist=[g getListOfObjectsSplittedAtPoint:p]))
+                if ( (nglist = [g getListOfObjectsSplittedAtPoint:p]) )
                 {   int i, ngCnt = [nglist count];
 
                     /* remove old object from list */
@@ -589,12 +590,12 @@
     if (gIsFilled)
         [self cache:affectedBounds];
     else
-        [self drawRect:affectedBounds];
+        [self setNeedsDisplayInRect:affectedBounds];    //[self drawRect:affectedBounds];
 }
 
-/* add an Point to object at point for VPath and VPolyLine
+/* add a Point to object at point for VPath and VPolyLine
  * created:  2005-06-26
- * modified: 2005-07-28
+ * modified: 2015-07-30 ([self setNeedsDisplayInRect:] (not -drawRect:))
  *
  * get objects splitted at point
  * save changes for undo
@@ -621,9 +622,7 @@
             if ([[layer list] indexOfObject:g] != NSNotFound) // object in this layer
             {
                 if ([(VPath*)g addPointAt:p])
-                {
                     break;
-                }
             }
         }
         [document setDirty:YES];
@@ -632,7 +631,7 @@
     if (gIsFilled)
         [self cache:affectedBounds];
     else
-        [self drawRect:affectedBounds];
+        [self setNeedsDisplayInRect:affectedBounds];    //[self drawRect:affectedBounds];
 }
 
 @end
